@@ -123,16 +123,24 @@ static int proc_phy_power_ctrl(struct ctl_table *ctl, int write,
 	struct phy_device *phydev = ndev->phydev;
 	int ret = 0;
 
+	pr_debug("%s: Enter: %s %s\n", __func__, dev->name, write ? "write" : "read");
+
 	if (!phydev && write)
 		return 0;
 
 	lctl = *ctl;
-	if (!write)
+	if (!write) {
 		*valp = ndev->phy_power;
-	else
+		pr_debug("%s: %s read hal_link_state %d\n", __func__, dev->name, *valp);
+	} else {
 		lctl.data = &val;
+		pr_debug("%s: %s write hal_link_state %d\n", __func__, dev->name, val);
+	}
 
 	ret = proc_dointvec(&lctl, write, buffer, lenp, ppos);
+	if (ret)
+		pr_err("%s: Error %d for %s %s\n", __func__, ret, dev->name, write ? "write" : "read");
+
 	if (write) {
 		switch (val) {
 		case HAL_LINK_STOP:
@@ -150,6 +158,8 @@ static int proc_phy_power_ctrl(struct ctl_table *ctl, int write,
 	if (ret)
 		*ppos = pos;
 
+	pr_debug("%s: Exit: %s %s ret %d\n", __func__, dev->name, write ? "write" : "read", ret);
+
 	return ret;
 }
 
@@ -467,7 +477,7 @@ struct sk_buff *dqnet_build_skb(struct fpm_buff *fb)
 
 	fpm_set_token_size(&fb->token, token_size);
 
-	skb = build_skb(fb->buf, size);
+	skb = __build_skb(fb->buf, (size + shinfo_size));
 	if (!skb)
 		return NULL;
 
@@ -1573,12 +1583,44 @@ static int dqnet_set_mtu(struct net_device *dev, int mtu)
 #endif
 		dqnet_fap_set_mtu(dev, mtu);
 	}
+
 	return status;
 }
 
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
+/*
+ * Handle device private ioctl requests to a device
+ */
+static int dqnet_siocdevprivate(struct net_device *dev, struct ifreq *rq,
+				void *user, int cmd)
+{
+	struct dqnet_netdev *ndev = netdev_priv(dev);
+	int status = -EOPNOTSUPP;
+
+	pr_debug("%s:-->\n",dev->name);
+
+	switch (cmd) {
+	case SIOCDEVPRIVATE:
+		if(get_user(ndev->mtu_pad,
+			   ((unsigned int *)rq->ifr_ifru.ifru_data)))
+			return -EFAULT;
+		if(dqnet_set_mtu(dev, dev->mtu))
+			return -EINVAL;
+		status = 0;
+		break;
+	default:
+		break;
+	}
+
+	pr_debug("%s:<--\n",dev->name);
+	return status;
+}
+#endif
+
 static int dqnet_mii_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
 	struct phy_device *phydev = dev->phydev;
+	struct mii_bus *bus = phydev->mdio.bus;
 	struct mii_ioctl_data *data = if_mii(rq);
 	u32 reg_num;
 	int ret;
@@ -1588,15 +1630,32 @@ static int dqnet_mii_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 		data->phy_id = phydev->mdio.addr;
 		break;
 	case SIOCGMIIREG:
-		reg_num = data->reg_num | (data->val_in << 16);
-		ret = mdiobus_read(phydev->mdio.bus, data->phy_id, reg_num);
+		if ((data->phy_id < PHY_MAX_ADDR) && bus->mdio_map[data->phy_id])
+			phydev = to_phy_device(&bus->mdio_map[data->phy_id]->dev);
+
+		if (phydev->drv && phydev->drv->read_mmd) {
+			ret = phy_read_mmd(phydev, data->val_in, data->reg_num);
+		} else {
+			reg_num = data->reg_num | (data->val_in << 16);
+			ret = mdiobus_read(bus, data->phy_id, reg_num);
+		}
+
 		if (ret < 0)
 			return ret;
+
 		data->val_out = ret;
 		break;
 	case SIOCSMIIREG:
-		reg_num = data->reg_num | (data->val_out << 16);
-		ret = mdiobus_write(phydev->mdio.bus, data->phy_id, reg_num, data->val_in);
+		if ((data->phy_id < PHY_MAX_ADDR) && bus->mdio_map[data->phy_id])
+			phydev = to_phy_device(&bus->mdio_map[data->phy_id]->dev);
+
+		if (phydev->drv && phydev->drv->write_mmd) {
+			ret = phy_write_mmd(phydev, data->val_out, data->reg_num, data->val_in);
+		} else {
+			reg_num = data->reg_num | (data->val_out << 16);
+			ret = mdiobus_write(bus, data->phy_id, reg_num, data->val_in);
+		}
+
 		if (ret < 0)
 			return ret;
 		break;
@@ -1612,7 +1671,9 @@ static int dqnet_mii_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 static int dqnet_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 {
 	int status = -EOPNOTSUPP;
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0))
 	struct dqnet_netdev *ndev = netdev_priv(dev);
+#endif
 
 	pr_debug("%s:-->\n",dev->name);
 
@@ -1623,6 +1684,7 @@ static int dqnet_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 		if (dev->phydev && dev->phydev->mdio.bus)
 			return dqnet_mii_ioctl(dev, rq, cmd);
 		break;
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0))
 	case SIOCDEVPRIVATE:
 		if(get_user(ndev->mtu_pad, ((unsigned int *)rq->ifr_ifru.ifru_data)))
 			return -EFAULT;
@@ -1630,6 +1692,7 @@ static int dqnet_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 			return -EINVAL;
 		status = 0;
 		break;
+#endif
 	default:
 		break;
 	}
@@ -2072,8 +2135,8 @@ int dqnet_open(struct net_device *dev)
 		goto err_release_dqms;
 
 	if (chan->open_count == 0) {
-		netif_napi_add(&napi_dev, &chan->napi, dqnet_poll_napi,
-			chan->napi.weight);
+		netif_napi_add(chan->demux ? &napi_dev : dev, &chan->napi, dqnet_poll_napi,
+			       chan->napi.weight);
 		napi_enable(&chan->napi);
 	}
 
@@ -2139,9 +2202,6 @@ int dqnet_open(struct net_device *dev)
 		}
 	}
 
-	dev->min_mtu = MIN_MTU_SIZE;
-	dev->max_mtu = MAX_MTU_SIZE;
-
 	goto done;
 
 err_remove_ndev:
@@ -2837,6 +2897,31 @@ int dqnet_probe(struct platform_device *pdev)
 			if (ndev->chan->type == DQNET_CHAN_TYPE_FAP_EXCEPT ||
 			    ndev->chan->type == DQNET_CHAN_TYPE_FAP_HOST)
 				ndev->link_stats = dqnet_fap_link_stats;
+			else {
+				status = bcm_nethook_register_hook(dev,
+					BCM_NETHOOK_TX_SKB,
+					TX_SKB_PRIO_DQNET_DROP_PRIV,
+					DQNET_DROP_PRIV_NAME,
+					dqnet_tx_drop_priv_cm_hook);
+				if (status) {
+					netdev_err(dev,
+						"Failed to register nethook tx: \"");
+					netdev_err(dev,
+						DQNET_DROP_PRIV_NAME"\"\n");
+					goto err_close_link;
+				}
+				status = bcm_nethook_enable_hook(dev,
+					BCM_NETHOOK_TX_SKB,
+					dqnet_tx_drop_priv_cm_hook,
+					true);
+				if (status) {
+					netdev_err(dev,
+						"Failed to enable nethook tx: \"");
+					netdev_err(dev,
+						DQNET_DROP_PRIV_NAME"\"\n");
+					goto err_close_link;
+				}
+			}
 
 			ndev->rpc_tunnel =
 				rpc_get_fifo_tunnel_id(ndev->rpc_name);
@@ -3083,6 +3168,9 @@ reg_done:
 		netdev_dbg(dev, "Device registered.\n");
 	}
 
+	dev->min_mtu = MIN_MTU_SIZE;
+	dev->max_mtu = MAX_MTU_SIZE;
+
 	dqnetstats_register_dev(ndev);
 
 	pr_info("%s: net device %s\n", MODULE_NAME, ndev->name);
@@ -3502,6 +3590,7 @@ struct net_device_ops dqnet_netdev_ops = {
 	.ndo_validate_addr	= eth_validate_addr,
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0))
 	.ndo_eth_ioctl		= dqnet_ioctl,
+	.ndo_siocdevprivate	= dqnet_siocdevprivate,
 #else
 	.ndo_do_ioctl		= dqnet_ioctl,
 #endif
